//$Id: JACCConfiguration.java,v 1.4 2005/04/03 18:28:28 oneovthafew Exp $
package org.hibernate.secure;

import javax.security.jacc.EJBMethodPermission;
import javax.security.jacc.PolicyConfiguration;
import javax.security.jacc.PolicyConfigurationFactory;
import javax.security.jacc.PolicyContextException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;

/**
 * Adds Hibernate permissions to roles via JACC
 * 
 * @author Gavin King
 */
public class JACCConfiguration {
	
	private static final Log log = LogFactory.getLog(JACCConfiguration.class);

	private final PolicyConfiguration policyConfiguration;

	public JACCConfiguration(String contextId) throws HibernateException {
		try {
			policyConfiguration = PolicyConfigurationFactory.getPolicyConfigurationFactory()
				.getPolicyConfiguration(contextId, false);
		}
		catch (ClassNotFoundException cnfe) {
			throw new HibernateException("JACC provider class not found", cnfe);
		}
		catch (PolicyContextException pce) {
			throw new HibernateException("policy context exception occurred", pce);
		}
	}
	
	public void addPermission(String role, String entityName, String action) {
		EJBMethodPermission permission = new EJBMethodPermission(entityName, action);
		if ( log.isDebugEnabled() ) {
			log.debug("adding permission to role " + role + ": " + permission);
		}
		try {
			policyConfiguration.addToRole(role, permission);
		}
		catch (PolicyContextException pce) {
			throw new HibernateException("policy context exception occurred", pce);
		}
	}
	

}
